defmodule MarkPropertyFavoriteContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.Favorites.Favorite

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    random_user = List.first(table)
    owner = List.last(table)

    {
      :ok,
      state
      |> Map.put(:random_email, random_user[:email])
      |> Map.put(:random_password, random_user[:password])
      |> Map.put(:owner_email, owner[:email])
      |> Map.put(:owner_password, owner[:password])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{ table_data: table } ->
    owner = Repo.get_by(User, email: state[:owner_email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^I am logged in as random user$/, fn state ->
    setup_session(state[:random_email], state[:random_password])
    {:ok, state}
  end

  and_ ~r/^I am logged in as owner$/, fn state ->
    setup_session(state[:owner_email], state[:owner_password])
    {:ok, state}
  end

  when_ ~r/^I visit the home page$/, fn state ->
    navigate_to("/")
    {:ok, state}
  end

  when_ ~r/^I visit the details page for "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    navigate_to("/properties/#{property.reference}")
    :timer.sleep(500)

    {:ok, state}
  end

  and_ ~r/^I click the favorite button$/, fn state ->
    button = find_element(:css, ".favorite-button")
    click(button)
    :timer.sleep(500)

    reference = attribute_value(button, "data-property-reference")
    property = Enum.find(state.advertisements, fn advert ->
      advert.reference == reference
    end)

    {:ok, state |> Map.put(:last_favorited_title, property.title)}
  end



  and_ ~r/^I click the favorite button for "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)
    button = find_element(:css, ".favorite-button[data-property-reference='#{property.reference}']")
    click(button)
    :timer.sleep(500)

    {:ok, state |> Map.put(:last_favorited_title, argument_one)}
  end

  then_ ~r/^I should see Property added to favorites message$/, fn state ->
    assert visible_in_page? ~r/Property added to favorites/
    {:ok, state}
  end

  then_ ~r/^I should not see a favorite button for "(?<argument_one>[^"]+)"$/,
  fn state, %{argument_one: argument_one} ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == argument_one
    end)

    elements = find_all_elements(:css, ".favorite-button[data-property-reference='#{property.reference}']")

    assert Enum.empty?(elements),
      "Expected no favorite button for property '#{argument_one}' but found #{length(elements)} button(s)"

    {:ok, state}
  end

  and_ ~r/^the property should be marked as favorite$/, fn state ->
    property = Enum.find(state.advertisements, fn advert ->
      advert.title == state.last_favorited_title
    end)

    button = find_element(:css, ".favorite-button[data-property-reference='#{property.reference}']")

    filled_star = "★"
    button_text = visible_text(button)
    assert button_text == filled_star, "Expected filled star (#{filled_star}) but got #{button_text}"

    favorited_attr = attribute_value(button, "data-favorited")
    assert favorited_attr == "true", "Expected data-favorited to be 'true' but got #{favorited_attr}"

    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end
end
