defmodule MarkPropertySoldRentedContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    first_user = List.first(table)
    second_user = if length(table) > 1, do: Enum.at(table, 1), else: nil

    {
      :ok,
      state
      |> Map.put(:first_email, first_user[:email])
      |> Map.put(:first_password, first_user[:password])
      |> Map.put(:second_email, second_user && second_user[:email])
      |> Map.put(:second_password, second_user && second_user[:password])
    }
  end

  given_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:first_email], state[:first_password])
    {:ok, state}
  end

  given_ ~r/^I am logged in as second user$/, fn state ->
    setup_session(state[:second_email], state[:second_password])
    {:ok, state}
  end

  and_ ~r/^the following properties exist$/, fn state, %{table_data: table} ->
    owner = Repo.get_by(User, email: state[:first_email])

    properties =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:properties, properties)
    }
  end

  when_ ~r/^I navigate to property details page$/, fn state ->
    property = List.first(state.properties)
    navigate_to("/properties/#{property.reference}")
    :timer.sleep(500)
    {:ok, state}
  end

  and_ ~r/^I click on drop-down menu and update status$/, fn state ->
    button = find_element(:css, ".status-dropdown-button")
    click(button)
    :timer.sleep(200)

    option = find_element(:css, "[data-status='unavailable']")
    click(option)
    :timer.sleep(500)

    {:ok, state}
  end

  and_ ~r/^I click on drop-down menu and set status to available$/, fn state ->
    button = find_element(:css, ".status-dropdown-button")
    click(button)
    :timer.sleep(200)
    option = find_element(:css, "[data-status='available']")
    click(option)
    :timer.sleep(500)
    {:ok, state}
  end

  then_ ~r/^The status should be updated in database and in page$/, fn state ->
    property = List.first(state.properties)
    updated_property = Repo.get(Property, property.id)

    assert updated_property.state == :unavailable
    assert visible_in_page? ~r/Rented/i

    {:ok, state}
  end

  then_ ~r/^The status should be set to available in database and in page$/, fn state ->
    property = List.first(state.properties)
    updated_property = Repo.get(Property, property.id)
    assert updated_property.state == :available
    assert visible_in_page? ~r/Available/i
    {:ok, state}
  end

  and_ ~r/^I should see a success message$/, fn state ->
    assert visible_in_page? ~r/Status updated successfully/i
    {:ok, state}
  end

  then_ ~r/^I should not see a drop-down menu$/, fn state ->
    elements = find_all_elements(:css, ".status-dropdown-button")
    assert Enum.empty?(elements)
    {:ok, state}
  end

  and_ ~r/^The status not updated in database$/, fn state ->
    property = List.first(state.properties)
    updated_property = Repo.get(Property, property.id)
    assert updated_property.state == :available
    {:ok, state}
  end

  when_ ~r/^I navigate to the homepage$/, fn state ->
    navigate_to("/")
    :timer.sleep(500)
    {:ok, state}
  end

  then_ ~r/^I should not see the unavailable property$/, fn state ->
    property = List.first(state.properties)
    property_title = property.title
    refute visible_in_page? ~r/#{property_title}/i
    {:ok, state}
  end

  then_ ~r/^I should see the available property$/, fn state ->
    property = List.first(state.properties)
    property_title = property.title
    assert visible_in_page? ~r/#{property_title}/i
    {:ok, state}
  end


  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end
end
