defmodule MyAdvertisementsContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Properties.Property
  alias PropTrackr.Accounts.User

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
    }
  end

  given_ ~r/^I am logged in$/, fn state ->
    setup_session(state[:email], state[:password])
    {:ok, state}
  end

  given_ ~r/^I want to insert the following data$/, fn state, %{table_data: table} ->
    data = List.first(table)
    {
      :ok,
      state
      |> Map.put(:data, data)
    }
  end

  given_ ~r/^I created given properties with photos$/, fn state ->
    navigate_to("/")
    click({:id, "add_advertisement"})
    fill_field({:id, "title"}, state[:data][:title])
    fill_field({:id, "description"}, state[:data][:description])
    select_dropdown("type", state[:data][:type])
    select_dropdown("property_type", state[:data][:property_type])
    fill_field({:id, "location"}, state[:data][:location])
    fill_field({:id, "area"}, state[:data][:area])
    fill_field({:id, "room_count"}, state[:data][:room_count])
    fill_field({:id, "floor"}, state[:data][:floor])
    fill_field({:id, "floor_count"}, state[:data][:floor_count])
    fill_field({:id, "price"}, state[:data][:price])

    file_input = find_element(:id, "property_photos")

    execute_script("""
      (function() {
        const input = document.getElementById('property_photos');
        const files = [
          new File(['test content'], 'test_photo_1.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_2.jpg', { type: 'image/jpeg' }),
          new File(['test content'], 'test_photo_3.jpg', { type: 'image/jpeg' })
        ];
        const dataTransfer = new DataTransfer();
        files.forEach(file => dataTransfer.items.add(file));
        input.files = dataTransfer.files;
        input.dispatchEvent(new Event('change', { bubbles: true }));
      })();
    """)

    click({:id, "submit_button"})
    property = Repo.get_by(Property, title: state[:data][:title])
    {:ok, state |> Map.put(:property, property)}
  end

  when_ ~r/^I navigate to my profile page$/, fn state ->
    navigate_to("/me")
    {:ok, state}
  end

  and_ ~r/^I click on My properties link$/, fn state ->
    click({:id, "my_properties"})
    {:ok, state}
  end

  then_ ~r/^I should be redirected to my properties page$/, fn state ->
    assert current_path() == "/me/properties"
    {:ok, state}
  end

  and_ ~r/^I should see a list of all my properties$/, fn state ->
    advertisement = state[:data]
    assert visible_in_page? ~r/#{advertisement.title}/
    assert visible_in_page? ~r/#{advertisement.description}/
    assert visible_in_page? ~r/#{advertisement.price}/
    assert visible_in_page? ~r/#{advertisement.room_count}/
    assert visible_in_page? ~r/#{advertisement.area}/
    {:ok, state}
  end

  given_ ~r/^I am logged in as "(?<email>[^"]+)"$/, fn state, %{email: email} ->
    user = Repo.get_by(User, email: email)
    setup_session(email, "password")
    {:ok, Map.put(state, :current_user, user)}
  end

  given_ ~r/^I log out$/, fn state ->
    navigate_to("/logout")
    {:ok, state}
  end

  and_ ~r/^I should not see other user's properties$/, fn state ->
    advertisement = state[:data]
    refute visible_in_page? ~r/#{advertisement.title}/
    refute visible_in_page? ~r/#{advertisement.description}/
    refute visible_in_page? ~r/#{advertisement.price}/
    refute visible_in_page? ~r/#{advertisement.room_count}/
    refute visible_in_page? ~r/#{advertisement.area}/
    {:ok, state}
  end

  and_ ~r/^I click to View more button$/, fn state ->
    view_state = state[:property]
    click({:id, "edit-#{view_state.reference}"})
    {:ok, state}
  end

  then_ ~r/^I should be redirected to property details page$/, fn state ->
    view_state = state[:property]
    assert current_path() == "/properties/#{view_state.reference}"
    {:ok, state}
  end

  and_ ~r/^I should see the property details$/, fn state ->
    advertisement = state[:property]
    assert visible_in_page? ~r/#{advertisement.title}/
    assert visible_in_page? ~r/#{advertisement.description}/
    assert visible_in_page? ~r/#{advertisement.location}/
    assert visible_in_page? ~r/#{advertisement.price}/
    assert visible_in_page? ~r/#{advertisement.room_count}/
    assert visible_in_page? ~r/#{advertisement.area}/
    assert visible_in_page? ~r/#{advertisement.type}/
    assert visible_in_page? ~r/#{advertisement.property_type}/
    user = Repo.get_by(User, id: advertisement.user_id)
    assert visible_in_page? ~r/#{user.name}/
    assert visible_in_page? ~r/#{user.surname}/
    assert visible_in_page? ~r/#{user.phone_number}/
    assert visible_in_page? ~r/#{user.email}/

    user = Repo.get_by(User, id: advertisement.user_id)
    assert visible_in_page? ~r/#{user.name}/
    assert visible_in_page? ~r/#{user.surname}/
    assert visible_in_page? ~r/#{user.phone_number}/
    assert visible_in_page? ~r/#{user.email}/

    assert visible_in_page?(~r/Edit Property/)
    assert visible_in_page?(~r/Delete Property/)

    elements = find_all_elements(:css, ".status-dropdown-button")
    refute Enum.empty?(elements)

    property_with_photos = Repo.preload(advertisement, :photos)
    Enum.each(property_with_photos.photos, fn photo ->

      assert find_element(:css, "img[src*='#{photo.filename}']")
      |> has_class?("w-full")

      assert find_element(:css, "button.thumbnail img[src*='#{photo.filename}']")
    end)
    {:ok, state}
  end

  defp setup_session(email, password) do
    navigate_to("/login")
    fill_field({:id, "email"}, email)
    fill_field({:id, "password"}, password)
    click({:id, "login_button"})
  end

  defp select_dropdown(drop_down_id, option) do
    find_element(:css, "##{drop_down_id} option[value='#{option}']") |> click()
  end
end
