defmodule SimilarPropertiesContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{}
  end

  scenario_finalize fn _status, _state ->
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^there exists following accounts$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)                            
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)

    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
      |> Map.put(:user_id, existing_user[:id])
    }
  end

  and_ ~r/^the following properties exist$/, fn state, %{table_data: table} ->
    owner = Repo.get_by(User, email: state[:email])

    advertisements =
      table
      |> Enum.map(fn details -> details |> Map.put(:reference, Ecto.UUID.generate()) end)
      |> Enum.map(fn details -> {Ecto.build_assoc(owner, :properties, details), details} end)
      |> Enum.map(fn {assoc, details} -> Property.changeset(assoc, details) end)
      |> Enum.map(fn changeset -> Repo.insert!(changeset) end)

    {
      :ok,
      state
      |> Map.put(:advertisements, advertisements)
    }
  end

  and_ ~r/^I go to the property page for "(?<title>[^"]+)"$/, fn state, %{title: title} ->
    advertisement = Enum.find(state.advertisements, fn advertisement -> advertisement.title == title end)
    navigate_to("/properties/#{advertisement.reference}")
    {:ok, state}
  end

  then_ ~r/^I should see a message that there are no similar properties$/, fn state ->
    assert visible_in_page? ~r/No similar properties found/
    {:ok, state}
  end

  then_ ~r/^I should see only 5 similar properties$/, fn state ->
    assert length(find_all_elements(:class, "similar-property")) == 5
    {:ok, state}
  end

  then_ ~r/^I should see the all similar properties$/, fn state ->
    assert length(find_all_elements(:class, "similar-property")) == length(state.advertisements) - 1
    state[:advertisements] |> Enum.each(fn advertisement ->
      assert visible_in_page? ~r/#{advertisement.title}/
      assert visible_in_page? ~r/#{:erlang.float_to_binary(advertisement.price, decimals: 2)}/
      assert visible_in_page? ~r/#{advertisement.location}/
    end)
    {:ok, state}
  end

  and_ ~r/^I click on View More for "(?<title>[^"]+)"$/, fn state, %{title: title} ->
    advertisement = Enum.find(state.advertisements, fn advertisement -> advertisement.title == title end)
    click({:id, "view-similar-property-#{advertisement.reference}"})
    {:ok, state}
  end

  then_ ~r/^I should be redirected to the property page for "(?<title>[^"]+)"$/, fn state, %{title: title} ->
    advertisement = Enum.find(state.advertisements, fn advertisement -> advertisement.title == title end)
    assert current_path() == "/properties/#{advertisement.reference}"
    assert visible_in_page? ~r/#{advertisement.title}/
    {:ok, state}
  end
end
