defmodule UserRegistrationContext do
  use WhiteBread.Context
  use Hound.Helpers
  alias PropTrackr.Accounts
  alias PropTrackr.Repo
  alias PropTrackr.Accounts.User

  scenario_starting_state fn _state ->
    Ecto.Adapters.SQL.Sandbox.checkout(PropTrackr.Repo)
    Ecto.Adapters.SQL.Sandbox.mode(PropTrackr.Repo, {:shared, self()})
    Hound.start_session()
    %{registered_users: []}
  end

  scenario_finalize fn _status, state ->
    # Enum.each(state[:registered_users], &Repo.delete/1)
    Ecto.Adapters.SQL.Sandbox.checkin(PropTrackr.Repo)
    Hound.end_session()
  end

  given_ ~r/^I want to register a new user$/, fn state ->
    navigate_to("/register")
    {:ok, state}
  end

  and_ ~r/^I have such user data$/, fn state, %{table_data: table} ->
    user_data = List.first(table)
    {:ok, state |> Map.put(:user_data, user_data)}
  end

  when_ ~r/^I fill a registration form with valid data$/, fn state ->
    fill_field({:id, "name"}, state[:user_data][:name])
    fill_field({:id, "surname"}, state[:user_data][:surname])
    formatted_date = formatDate(state[:user_data][:birth_date])
    fill_field({:id, "birth_date"}, formatted_date)
    fill_field({:id, "phone_number"}, state[:user_data][:phone_number])
    fill_field({:id, "email"}, state[:user_data][:email])
    fill_field({:id, "password"}, state[:user_data][:password])
    fill_field({:id, "confirm_password"}, state[:user_data][:confirm_password])
    {:ok, state}
  end

  when_ ~r/^I click submit$/, fn state ->
    click({:id, "register_button"})
    {:ok, state}
  end

  then_ ~r/^I should see a success message$/, fn state ->
    assert visible_in_page? ~r/User created successfully./
    {:ok, state}
  end

  and_ ~r/^the user should be saved in the database$/, fn state ->
    assert Repo.get_by(User, email: state[:user_data][:email])
    {:ok, state}
  end

  and_ ~r/^I should be redirected to the main page$/, fn state ->
    assert current_path() == "/"
    {:ok, state}
  end

  when_ ~r/^I fill a registration form with invalid phone number$/, fn state ->
    fill_field({:id, "name"}, state[:user_data][:name])
    fill_field({:id, "surname"}, state[:user_data][:surname])
    formatted_date = formatDate(state[:user_data][:birth_date])
    fill_field({:id, "birth_date"}, formatted_date)
    fill_field({:id, "phone_number"}, state[:user_data][:phone_number])
    fill_field({:id, "email"}, state[:user_data][:email])
    fill_field({:id, "password"}, state[:user_data][:password])
    fill_field({:id, "confirm_password"}, state[:user_data][:confirm_password])
    {:ok, state}
  end

  then_ ~r/^I should see an error message about the phone number$/, fn state ->
    assert visible_page_text() =~ "must be between 3 and 15 digits, without '+' sign"
    {:ok, state}
  end

  and_ ~r/^the user should not be saved in the database$/, fn state ->
    refute Repo.get_by(User, email: state[:user_data][:email])
    {:ok, state}
  end

  and_ ~r/^I should still see the registration form$/, fn state ->
    assert current_path() == "/register"
    {:ok, state}
  end

  when_ ~r/^I fill a registration form with invalid date$/, fn state ->
    fill_field({:id, "name"}, state[:user_data][:name])
    fill_field({:id, "surname"}, state[:user_data][:surname])
    formatted_date = formatDate(state[:user_data][:birth_date])
    fill_field({:id, "birth_date"}, formatted_date)
    fill_field({:id, "phone_number"}, state[:user_data][:phone_number])
    fill_field({:id, "email"}, state[:user_data][:email])
    fill_field({:id, "password"}, state[:user_data][:password])
    fill_field({:id, "confirm_password"}, state[:user_data][:confirm_password])
    {:ok, state}
  end

  then_ ~r/^I should see an error message about the birth date$/, fn state ->
    assert visible_page_text() =~ "Birth date cannot be in the future" or
    assert visible_page_text() =~ "Date is too past" or
    assert visible_page_text() =~ "is invalid"
    {:ok, state}
  end

  given_ ~r/^there exists the following account$/, fn state, %{table_data: table} ->
    table
    |> Enum.map(fn user_details -> User.changeset(%User{}, user_details) end)
    |> Enum.each(fn changeset -> Repo.insert!(changeset) end)

    existing_user = List.first(table)
    {
      :ok,
      state
      |> Map.put(:email, existing_user[:email])
      |> Map.put(:password, existing_user[:password])
    }
  end

  when_ ~r/^I fill a registration form with existing email$/, fn state ->
    fill_field({:id, "name"}, state[:user_data][:name])
    fill_field({:id, "surname"}, state[:user_data][:surname])
    formatted_date = formatDate(state[:user_data][:birth_date])
    fill_field({:id, "birth_date"}, formatted_date)
    fill_field({:id, "phone_number"}, state[:user_data][:phone_number])
    fill_field({:id, "email"}, state[:user_data][:email])
    fill_field({:id, "password"}, state[:user_data][:password])
    fill_field({:id, "confirm_password"}, state[:user_data][:confirm_password])
    {:ok, state}
  end

  then_ ~r/^I should see an error message about the email$/, fn state ->
    assert visible_page_text() =~ "An account with this email already exists."
    {:ok, state}
  end

  when_ ~r/^I fill a registration form with mismatched passwords$/, fn state ->
    fill_field({:id, "name"}, state[:user_data][:name])
    fill_field({:id, "surname"}, state[:user_data][:surname])
    formatted_date = formatDate(state[:user_data][:birth_date])
    fill_field({:id, "birth_date"}, formatted_date)
    fill_field({:id, "phone_number"}, state[:user_data][:phone_number])
    fill_field({:id, "email"}, state[:user_data][:email])
    fill_field({:id, "password"}, state[:user_data][:password])
    fill_field({:id, "confirm_password"}, state[:user_data][:confirm_password])
    {:ok, state}
  end

  then_ ~r/^I should see an error message about the passwords$/, fn state ->
    assert visible_page_text() =~ "Passwords do not match."
    {:ok, state}
  end

  defp formatDate(date) do
    [year, month, day] = String.split(date, "-")
    formatted_date = "#{month}-#{day}-#{year}"
    formatted_date
  end
end
