defmodule PropTrackr.Search do
  use Ecto.Schema
  import Ecto.Changeset
  alias PropTrackr.Repo
  alias PropTrackr.Properties.Property
  alias PropTrackr.NotInterested

  import Ecto.Query

  schema "searches" do
    field :type, Ecto.Enum, values: [:rent, :sell, :any]
    field :min_price, :float
    field :max_price, :float

    field :min_rooms, :integer
    field :max_rooms, :integer

    field :location, :string
    field :areas, {:array, :string}

    belongs_to :user, PropTrackr.Accounts.User

    timestamps()
  end

  def create_query(struct, current_user) do
    areas = if Map.has_key?(struct, :areas) do
      struct.areas
    else
      []
    end

    locations = if areas == [] do
      []
    else
      Enum.map(areas, fn area -> area <> ", " <> struct.location end)
    end

    current_user_id = if current_user != nil do
      current_user.id
    else
      nil
    end

    query = from(
      p in Property,
      left_join: ni in NotInterested,
      on: ni.property_id == p.id and ni.user_id == ^(current_user_id || 0),
      order_by: [asc: not is_nil(ni.id), asc: p.state, asc: p.inserted_at],
      where: p.price >= ^struct.min_price,
      where: p.price <= ^struct.max_price,
      where: p.room_count >= ^struct.min_rooms,
      where: p.room_count <= ^struct.max_rooms,
      where: p.state != :unavailable,
      preload: [:photos],
      select: p,
    )

    query = if locations == [] do
      query
    else
      query |> where([p], p.location in ^locations)
    end

    query = if struct.type == :any do
      query
    else
      query |> where(type: ^struct.type)
    end

    query
  end

  def changeset(struct, params \\ %{}) do
    struct
    |> cast(params, [:type, :location, :min_price, :max_price, :min_rooms, :max_rooms, :areas])
    |> validate_required([:type, :location, :min_price, :max_price, :min_rooms, :max_rooms])
    |> validate_minmax(:min_price, :max_price)
    |> validate_minmax(:min_rooms, :max_rooms)
    |> cast_areas()
  end

  defp validate_minmax(changeset, min_key, max_key) do
    min = get_field(changeset, min_key)
    max = get_field(changeset, max_key)

    cond do
      min < 0 -> add_error(changeset, min_key, "cannot be negative")
      min > max -> add_error(changeset, min_key, "cannot be greater than #{max_key}")
      true -> changeset
    end
  end
  
  defp cast_areas(changeset) do
    areas = get_field(changeset, :areas)
    if areas == nil do
      put_change(changeset, :areas, [])
    else
      changeset
    end
  end
end
