defmodule PropTrackrWeb.InterestControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Properties.Property
  alias PropTrackr.NotInterested
  alias PropTrackr.Repo
  import Ecto.Query

  @password "password123"

  setup do
    owner = %User{
      name: "Property", surname: "Owner",
      birth_date: ~D[2000-01-01], phone_number: "111",
      email: "property.owner@gmail.com",
      password: @password, confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    } |> Repo.insert!()

    random_user = %User{
      name: "Random", surname: "User",
      birth_date: ~D[2000-01-01], phone_number: "000",
      email: "random.user@gmail.com",
      password: @password, confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    } |> Repo.insert!()

    property = %Property{
      reference: Ecto.UUID.generate(),
      title: "Test Property", description: "Test Description",
      type: :sell, property_type: :house, state: :available,
      location: "Test Location", room_count: 3, area: 100.0,
      floor: 2, floor_count: 5, price: 500000.0,
      user_id: owner.id
    } |> Repo.insert!()

    {:ok, %{owner: owner, random_user: random_user, property: property}}
  end

  test "authenticated user can mark property as not interested", %{conn: conn, random_user: user, property: property} do
    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/#{property.reference}/interest")

    assert json_response(conn, 200)["message"] == "Marked as not interested"

    not_interested = Repo.one(from n in NotInterested,
      where: n.user_id == ^user.id and n.property_id == ^property.id)
    assert not_interested != nil
  end

  test "authenticated user cannot mark same property as not interested twice", %{conn: conn, random_user: user, property: property} do
    Repo.insert!(%NotInterested{user_id: user.id, property_id: property.id})

    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/#{property.reference}/interest")

    assert json_response(conn, 422)["error"] == "Already marked as not interested"

    count = Repo.one(from n in NotInterested,
      where: n.user_id == ^user.id and n.property_id == ^property.id,
      select: count(n.id))
    assert count == 1
  end

  test "owner cannot mark own property as not interested", %{conn: conn, owner: owner, property: property} do
    conn = conn |> setup_session(owner)
    conn = post(conn, ~p"/api/properties/#{property.reference}/interest")

    assert json_response(conn, 403)["error"] == "Cannot mark own property as not interested"

    not_interested = Repo.one(from n in NotInterested,
      where: n.user_id == ^owner.id and n.property_id == ^property.id)
    assert not_interested == nil
  end

  test "unauthenticated user cannot mark property as not interested", %{conn: conn, property: property} do
    initial_count = Repo.one(from n in NotInterested, select: count(n.id))
    conn = post(conn, ~p"/api/properties/#{property.reference}/interest")

    assert json_response(conn, 401)["error"] == "You must be logged in"
    final_count = Repo.one(from n in NotInterested, select: count(n.id))
    assert initial_count == final_count
  end

  test "cannot mark non-existent property as not interested", %{conn: conn, random_user: user} do
    conn = conn |> setup_session(user)
    conn = post(conn, ~p"/api/properties/non-existent/interest")

    assert json_response(conn, 404)["error"] == "Property not found"
  end

  test "authenticated user can mark property as interested again", %{conn: conn, random_user: user, property: property} do
    Repo.insert!(%NotInterested{user_id: user.id, property_id: property.id})

    conn = conn |> setup_session(user)
    conn = delete(conn, ~p"/api/properties/#{property.reference}/interest")

    assert json_response(conn, 200)["message"] == "Marked as interested"

    not_interested = Repo.one(from n in NotInterested,
      where: n.user_id == ^user.id and n.property_id == ^property.id)
    assert not_interested == nil
  end

  test "authenticated user cannot mark interested property as interested again", %{conn: conn, random_user: user, property: property} do
    conn = conn |> setup_session(user)
    conn = delete(conn, ~p"/api/properties/#{property.reference}/interest")

    assert json_response(conn, 404)["error"] == "Already marked as interested"
  end

  test "unauthenticated user cannot mark property as interested", %{conn: conn, random_user: user, property: property} do
    Repo.insert!(%NotInterested{user_id: user.id, property_id: property.id})
    initial_count = Repo.one(from n in NotInterested, select: count(n.id))

    conn = delete(conn, ~p"/api/properties/#{property.reference}/interest")

    assert json_response(conn, 401)["error"] == "You must be logged in"
    final_count = Repo.one(from n in NotInterested, select: count(n.id))
    assert initial_count == final_count
  end

  test "deleting property removes associated not interested marks", %{conn: _conn, random_user: user, property: property} do
    Repo.insert!(%NotInterested{user_id: user.id, property_id: property.id})
    Repo.delete!(property)

    not_interested = Repo.one(from n in NotInterested,
      where: n.user_id == ^user.id and n.property_id == ^property.id)
    assert not_interested == nil
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end
 end
