defmodule PropTrackrWeb.PasswordControllerTest do
  use PropTrackrWeb.ConnCase
  alias PropTrackr.Accounts.User
  alias PropTrackr.Repo

  @user_credentials %{email: "test.user@gmail.com", password: "1234", new_password: "new_password"}
  @password "1234"

  setup do
    user = %User{
      name: "Test",
      surname: "User",
      birth_date: ~D[2000-01-01],
      phone_number: "000",
      bio: "Yo",
      email: "test.user@gmail.com",
      password: @password,
      confirm_password: @password,
      hashed_password: Bcrypt.hash_pwd_salt(@password)
    }
    user = Repo.insert!(user)

    {:ok, %{user: user}}
  end

  test "Authenticated user should be able to change their password with correct current credentials and matching new passwords", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    old_hashed_password = user.hashed_password

    conn = conn
    |> post("/me/password",
      current_password: @user_credentials[:password],
      new_password: @user_credentials[:new_password],
      new_password_confirmation: @user_credentials[:new_password])

    assert redirected_to(conn) == "/login"
    conn = get conn, redirected_to(conn)
    assert html_response(conn, 200)
    assert get_flash(conn, :info) =~ ~r/Password changed successfully! Please log in again/
    assert get_session(conn, :user_id) == nil

    updated_user = Repo.get(User, user.id)

    assert updated_user.hashed_password != old_hashed_password

    assert Bcrypt.verify_pass(@user_credentials[:new_password], updated_user.hashed_password)
    refute Bcrypt.verify_pass(@user_credentials[:password], updated_user.hashed_password)
  end

  test "Authenticated user should receive an error if the current password is invalid", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    conn = conn
           |> post("/me/password",
                   current_password: "obviously_invalid",
                   new_password: @user_credentials[:new_password],
                   new_password_confirmation: @user_credentials[:new_password])
    assert html_response(conn, 200)
    assert get_flash(conn, :error) =~ ~r/Invalid current password!/
  end

  test "Authenticated user should receive an error if the new password does not match the new password confirmation", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    conn = conn
           |> post("/me/password",
                   current_password: @user_credentials[:password],
                   new_password: @user_credentials[:new_password],
                   new_password_confirmation: "obviously_does_not_match")
    assert html_response(conn, 200)
    assert get_flash(conn, :error) =~ ~r/New passwords don't match!/
  end

  test "Authenticated user should receive an error if the new password is the same as the old password", %{conn: conn, user: user} do
    conn = conn |> setup_session(user)
    conn = conn
           |> post("/me/password",
                   current_password: @user_credentials[:password],
                   new_password: @user_credentials[:password],
                   new_password_confirmation: @user_credentials[:password])
    assert html_response(conn, 200)
    assert get_flash(conn, :error) =~ ~r/New password can't be the same as the old password!/
  end

  defp setup_session(conn, user) do
    conn = conn |> post("/login", email: user.email, password: user.password)
    conn = get conn, redirected_to(conn)
    conn
  end
end
